

/* --
  text
  -- */
.text {
  font-size: 14px;
  font-family: "Helvetica", "Arial", "Roboto", sans-serif;
  color: var(--nordic);
}
.text--blue-accent {
  color: var(--slate-grey);
}
.text--small {
  font-size: 80%;
}
.text--24px {
  font-size: 24px !important;
}
.text--20px {
  font-size: 20px !important;
}
.text--18px {
  font-size: 18px !important;
}
.text--16px {
  font-size: 16px !important;
}
.text--14px {
  font-size: 14px !important;
}
h1, h2, h3, h4, h5, h6 {
  font-weight: 700;
  padding: 0;
  margin: 0;
  line-height: 1em;
  padding-bottom: 0.5em;
}
h1,
.text--h1 {
  font-size: 26px;
}
h2,
.text--h2 {
  font-size: 20px;
}
h3,
.text--h3 {
  font-size: 18px;
}
h4,
.text--h4 {
  font-size: 16px;
}
h5,
.text--h5 {
  font-size: 14px;
  line-height: 18px;
}
h6,
.text--h6 {
  font-size: 12px;
  line-height: 18px;
}

.text--500 {
  font-weight: 500 !important;
}
.text--600 {
  font-weight: 600 !important;
}
.text--700 {
  font-weight: 700 !important;
}

.list__text-label {
  padding: 4px 12px 4px 4px;
  line-height: 130%;
  font-weight: 600;
}

.tag {
  background-color: var(--mint-blue);
  padding: 2px 6px !important;
  font-weight: 400 !important;
  border-radius: 3px !important;
}



